@@ -160,10 +160,10 @@ class AmiEntry:
         """set wikidataID, id must be Pddd... or Q... """
         if idx is None or not re.match("[PQ]\\d+", idx):
             raise AMIDictError(f"wikidata id {idx} must be Qddd... or Pddd...")
-        self.set_attribute(WIKIDATA, idx)
+        self.set_attribute(WIKIDATA_ID, idx)
 
     def wikidata_id(self):
-        return self.get_attribute_value(WIKIDATA)
+        return self.get_attribute_value(WIKIDATA_ID)
 
     def set_description(self, desc):
         """set description attribute, can be anything"""
@@ -186,6 +186,7 @@ class AmiEntry:
             # msg = f"ATT {attribut}"
             assert type(attribut) is str
             assert attribut in AmiEntry.ALLOWED_ATTS, f"attribute {attribut} is not allowed in <entry>"
+            self.validate_attribute(AmiEntry.WIKIDATA_A)
 
     def check_valid_children(self):
         if False:
@@ -202,6 +203,22 @@ class AmiEntry:
             ami_synonyms.append(AmiSynonym.create_from_element(synonym_element))
         return ami_synonyms
 
+    def validate_attribute(self, att_name):
+        if not att_name:
+            logger.warning(f"attribute name is None")
+            return
+        value = self.element.attrib.get(att_name)
+        if att_name == WIKIDATA_ID:
+            self.validate_wikidata_id(value)
+
+    def validate_wikidata_id(self, value):
+        if not value:
+            return None
+        mm = re.match("(Q|P)\d{1,11}", value)
+        if not mm:
+            logging.warning(f"{WIKIDATA_ID} bad value: {value}")
+        return mm
+
 
 class AmiDictionary:
     """wrapper for an ami dictionary including search flags
