@@ -1,39 +1,37 @@
 """ Mainly for converting PDF to HTML and SVG """
 import argparse
+import copy
 import logging
 import os.path
-import sys
-import copy
+import re
+import statistics
+import traceback
+from io import BytesIO, StringIO
+from pathlib import Path
+from typing import Container
 
 import lxml
 import lxml.html
-from lxml import etree
-from lxml.builder import E
-import statistics
-from pathlib import Path
-from typing import Container
-from io import BytesIO, StringIO
-import re
-import traceback
 import numpy as np
-from pdfminer.converter import TextConverter, XMLConverter, HTMLConverter
-from pdfminer.layout import LAParams
-from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
-from pdfminer.pdfpage import PDFPage
-from pdfminer.image import ImageWriter
-from pdfminer.layout import LTImage
 import pdfplumber
 from PIL import Image
-
+from lxml import etree
+from lxml.builder import E
+from pdfminer.converter import TextConverter, XMLConverter, HTMLConverter
+from pdfminer.image import ImageWriter
+from pdfminer.layout import LAParams
+from pdfminer.layout import LTImage
+from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
+from pdfminer.pdfpage import PDFPage
 from sklearn.linear_model import LinearRegression
 
-# local
-from py4ami.bbox_copy import BBox  # this is horrid, but I don't have a library
-from py4ami.util import Util, AbstractArgs
+from py4ami.ami_html import H_SPAN, H_A, A_HREF, H_TR, H_TD, H_TABLE, H_THEAD, H_TBODY
 from py4ami.ami_html import HtmlUtil, CSSStyle, HtmlTree, AmiSpan
 from py4ami.ami_html import STYLE, BOLD, ITALIC, FONT_FAMILY, FONT_SIZE, FONT_WEIGHT, FONT_STYLE, STROKE, FILL, TIMES, \
     CALIBRI, FONT_FAMILIES, H_DIV, H_BODY
-from py4ami.ami_html import H_SPAN, H_A, A_HREF, H_TR, H_TD, H_TABLE, H_THEAD, H_TBODY
+# local
+from py4ami.bbox_copy import BBox  # this is horrid, but I don't have a library
+from py4ami.util import Util, AbstractArgs
 
 # text attributes
 FACT = 2.8
@@ -120,6 +118,7 @@ class AmiPage:
     MESSY (because PDF is horrible)
     """
     CONTENT_RANGES = [[56, 999], [45, 780]]
+    DEFAULT_BBOX = BBox(xy_ranges=[[0, 9999], [0, 9999]])
 
     def __init__(self):
         # a mess because it started with SVG and new we are adding PDF
@@ -511,6 +510,43 @@ class AmiPage:
     def create_page_from_pdf_html(cls, path):
         logging.error("NOT YET WRITTEN")
 
+    @classmethod
+    def create_html_pages(cls,
+                          bbox=DEFAULT_BBOX,
+                          input_pdf=None,
+                          output_dir=None,
+                          output_stem=None,
+                          range_list=range(1,9999999)):
+        """create HTML pages from PDF
+        USED
+        uses pdfminer routines (AmiPage.chars_to_spans)
+        will need further tuning to generate structured HTML
+        :param bbox: clip page (default is none)
+        :param inout_pdf: required PDF
+        :param output_dir: output dicrectory
+        :param output_stem: output filestem
+        :param page_nos: list of 2-tuples containing allowed ranges (e.g.  [(2,3), (5, 12)]
+        """
+        if not input_pdf or not Path(input_pdf).exists():
+            logging.logger.error(f"must have not-null, existing pdf {input_pdf} ")
+
+        if not output_dir.exists():
+            output_dir.mkdir()
+        with pdfplumber.open(input_pdf) as pdf:
+            page_count = len(pdf.pages)
+        for page_no in range(page_count):
+            print(f"testing page {page_no}")
+        # for page_no in page_nos:
+            if not Util.range_list_contains_int(page_no, range_list):
+                continue
+            print(f"accept page {page_no}")
+            html = AmiPage.chars_to_spans(bbox, input_pdf, page_no)
+            output_html = Path(output_dir, f"{output_stem}_{page_no}.html")
+            with open(output_html, "wb") as f:
+                f.write(lxml.etree.tostring(html))
+                print(f" wrote html {output_html}")
+                # assert output_html.exists()
+
 
 class AmiSect:
     """Transformation of an Html Page to sections
@@ -685,64 +721,89 @@ CONVERT = "convert"
 FLOW = "flow"
 FOOTER = "footer"
 HEADER = "header"
+
 INDIR = "indir"
+INFORM = "inform"
 INPATH = "inpath"
+INSTEM = "instem"
+
 MAXPAGE = "maxpage"
+
 OUTDIR = "outdir"
-OUTPATH = "outpath"
 OUTFORM = "outform"
+OUTPATH = "outpath"
 OUTSTEM = "outstem"
 
+PAGES = "pages"
+PDF2HTML = "pdf2html"
+
 # FORMAT = "fmt"
 IMAGEDIR = "imagedir"
 RESOLUTION = "resolution"
 TEMPLATE = "template"
 
 
-
-
 class PDFArgs(AbstractArgs):
     def __init__(self):
         """arg_dict is set to default"""
         super().__init__()
         self.convert = DEFAULT_CONVERT
-        self.out_fmt = DEFAULT_CONVERT
         self.html = None
+
+        self.footer = None
+        self.header = None
+
         self.indir = None
+        self.inform = 'PDF'
         self.inpath = None
+        self.instem = 'fulltext'
+
         self.maxpage = DEFAULT_MAXPAGES
+
         self.outdir = None
+        self.outform = DEFAULT_CONVERT
         self.outpath = None
         self.outstem = None
+
+        self.pages = None
+
+        self.pdf2html = None
         self.raw_html = None
         self.flow = None
         self.unwanteds = None
 
-    @property
-    def module_stem(self):
-        return "ami_pdf"
-
-    def create_arg_parser(self):
+    def add_arguments(self):
         """creates adds the arguments for pyami commandline
 
         """
-        print(f"sys.argv {sys.argv}")
-        self.parser = argparse.ArgumentParser(description='PDF parsing')
-        self.parser.add_argument("--convert", type=str, choices=[], help="conversions (NYI)")
+        if self.parser is None:
+            self.parser = argparse.ArgumentParser()
+        self.parser.description='PDF parsing. (still evolving) allows for various inputs and outputs and may become a parent parser. Currently either use ' \
+                                'a) inpath and outpath' \
+                                'b) inpath and outdir/outstem/outform (especially where splitting to pages)' \
+                                'c)'
+        # self.parser.add_argument("--convert", type=str, choices=[], help="conversions (NYI)")
         self.parser.add_argument("--debug", type=str, choices=DEBUG_OPTIONS, help="debug these during parsing (NYI)")
         self.parser.add_argument("--flow", type=bool, nargs=1, help="create flowing HTML (heuristics)", default=True)
         self.parser.add_argument("--footer", type=float, nargs=1, help="top margin (clip everythimg above)", default=80)
         self.parser.add_argument("--header", type=float, nargs=1, help="bottom margin (clip everything below", default=80)
         self.parser.add_argument("--imagedir", type=str, nargs=1, help="output images to imagedir")
-        self.parser.add_argument("--indir", type=str, nargs=1, help="input directory")
-        self.parser.add_argument("--inpath", type=str, nargs=1, help="input file or (NYI) url")
+
+        self.parser.add_argument("--indir", type=str, nargs=1, help="input directory (might be calculated from inpath)")
+        self.parser.add_argument("--inform", type=str, nargs="+", help="input formats (might be calculated from inpath)")
+        self.parser.add_argument("--inpath", type=str, nargs=1, help="input file or (NYI) url; might be calculated from dir/stem/form")
+        self.parser.add_argument("--instem", type=str, nargs=1, help="input stem (e.g. 'fulltext'); maybe calculated from 'inpath`")
+
         self.parser.add_argument("--maxpage", type=int, nargs=1, help="maximum number of pages (will be deprecated, use 'pages')", default=self.arg_dict.get(MAXPAGE))
-        self.parser.add_argument("--pages", type=tuple, nargs="+", help="NYI (reads (1,3) or [(1, 3), (5, 10)]", default=[])
+
         self.parser.add_argument("--outdir", type=str, nargs=1, help="output directory")
-        self.parser.add_argument("--outstem", type=str, nargs=1, help="output file", default="fulltext.flow")
+        self.parser.add_argument("--outpath", type=str, nargs=1, help="output path (can be calculated from dir/stem/form)")
+        self.parser.add_argument("--outstem", type=str, nargs=1, help="output stem", default="fulltext.flow")
         self.parser.add_argument("--outform", type=str, nargs=1, help="output format ", default="html")
-        self.parser.add_argument("--resolution", type=int, nargs=1, help="resolution of output images (if imagedir)",
-                                 default=400)
+
+        self.parser.add_argument("--pdf2html", type=str, choices=['pdfminer', 'pdfplumber'], help="convert PDF to html", default='pdfminer')
+        self.parser.add_argument("--pages", type=int, nargs="+", help="(reads [3,5,9,12] -> 3-5 exclusive, 9-12 exclusive", default=all)
+        self.parser.add_argument("--resolution", type=int, nargs=1, help="resolution of output images (if imagedir)", default=400)
         self.parser.add_argument("--template", type=str, nargs=1, help="file to parse specific type of document (NYI)")
         return self.parser
 
@@ -771,6 +832,17 @@ class PDFArgs(AbstractArgs):
         self.check_output()
         self.calculate_headers_footers()
 
+        if self.pdf2html:
+            self.check_output()
+            range_list = self.create_range_list()
+            AmiPage.create_html_pages(
+                          bbox=AmiPage.DEFAULT_BBOX,
+                          input_pdf=self.inpath,
+                          output_dir=self.outdir,
+                          output_stem=self.outstem,
+                          range_list=range_list
+            )
+
         # self.convert_write()
 
     def check_input(self):
@@ -800,9 +872,9 @@ class PDFArgs(AbstractArgs):
             self.outstem = self.arg_dict.get(OUTSTEM)
             if not self.outstem:
                 raise FileNotFoundError(f"output file not given and cannot be generated")
-            if not self.out_fmt:
+            if not self.outform:
                 raise FileNotFoundError(f"no output format given")
-            self.outpath = Path(self.outdir, f"{self.outstem}.{self.out_fmt}")
+            self.outpath = Path(self.outdir, f"{self.outstem}.{self.outform}")
             print(f"outpath {self.outpath}")
         else:
             print(f"outdir {self.outdir}")
@@ -814,24 +886,31 @@ class PDFArgs(AbstractArgs):
             print(f"output dir {self.outdir}")
         return True
 
-
-
-
     def read_arg_dict(self):
         self.flow = self.arg_dict.get(FLOW) is not None
+
         self.footer = self.arg_dict.get(FOOTER)
         if not self.footer:
             self.footer = 80
         self.header = self.arg_dict.get(HEADER)
         if not self.header:
             self.header = 80
+
         self.indir = self.arg_dict.get(INDIR)
+        self.inform = self.arg_dict.get(INFORM)
         self.inpath = self.arg_dict.get(INPATH)
+        self.instem = self.arg_dict.get(INSTEM)
+
         self.maxpage = self.arg_dict.get(MAXPAGE)
+
         self.outdir = self.arg_dict.get(OUTDIR)
-        self.out_fmt = self.arg_dict.get(OUTFORM)
+        self.outform = self.arg_dict.get(OUTFORM)
+        self.outpath = self.arg_dict.get(OUTPATH)
         self.outstem = self.arg_dict.get(OUTSTEM)
-        self.convert = self.arg_dict.get(CONVERT)
+
+        self.pages = self.arg_dict.get(PAGES)
+
+        self.pdf2html = self.arg_dict.get(PDF2HTML)
 
             # self.convert_write(maxpage=maxpage, outdir=outdir, outstem=outstem, fmt=fmt, inpath=inpath, flow=True)
 
@@ -906,12 +985,21 @@ class PDFArgs(AbstractArgs):
         arg_dict[FLOW] = True
         arg_dict[FOOTER] = 80
         arg_dict[HEADER] = 80
+
         arg_dict[INDIR] = None
+        arg_dict[INFORM] = None
         arg_dict[INPATH] = None
+        arg_dict[INSTEM] = None
+
         arg_dict[MAXPAGE] = 5
+
         arg_dict[OUTDIR] = None
         arg_dict[OUTFORM] = "html"
+        arg_dict[OUTPATH] = None
         arg_dict[OUTSTEM] = None
+
+        arg_dict[PAGES] = None
+        arg_dict[PDF2HTML] = None
         arg_dict[FLOW] = True
         return arg_dict
 
@@ -962,7 +1050,7 @@ class PDFArgs(AbstractArgs):
     def convert_write(self):
         print(f"==============CONVERT================")
         self.process_args()
-        self.raw_html = PDFArgs.convert_pdf(path=self.inpath, fmt=self.out_fmt, maxpages=self.maxpage)
+        self.raw_html = PDFArgs.convert_pdf(path=self.inpath, fmt=self.outform, maxpages=self.maxpage)
 
         if self.flow:
             self.html = self.tidy_flow()
@@ -1039,7 +1127,8 @@ class PDFArgs(AbstractArgs):
                    r"(.*)" \
                    r")"
 
-        if result := re.compile(refregex).search(text):
+        result = re.compile(refregex).search(text)
+        if result:
             # print(f"matched: {result.group(1)} {result.group(2)}, {result.group(3)} {result.groups()}")
             elem0 = lxml.etree.SubElement(par, H_SPAN)
             elem0.text = result.group(1)
@@ -1069,6 +1158,21 @@ class PDFArgs(AbstractArgs):
 
             # print(f"par {lxml.etree.tostring(par)}")
 
+    @property
+    def module_stem(self):
+        """name of module"""
+        return Path(__file__).stem
+
+    def create_range_list(self):
+        """makes list of ranges from pairs on numbers"""
+        range_list = range(1,999)
+        if type(self.pages) is list:
+            range_list = []
+            ll = list(map(int, self.pages))
+            for i in range(0, len(ll), 2):
+                range_list.append(ll[i:i + 2])
+        return range_list
+
 
 class PDFDebug:
     def __init__(self):
@@ -1120,18 +1224,21 @@ class PDFDebug:
         print(f"words {len(page.extract_words())}", end=" | ")
 
     def print_lines(self, page):
-        if (n_line := len(page.lines)) > 0:
+        n_line = len(page.lines)
+        if n_line > 0:
             print(f"lines {n_line}", end=" | ")
 
     def print_rects(self, page, debug=False):
-        if n_rect := len(page.rects) > 0:
+        n_rect = len(page.rects)
+        if n_rect > 0:
             print(f"rects {n_rect}", end=" | ")
             if debug:
                 for rect in page.rects[:self.max_rect]:
                     print(f"rect (({rect['x0']},{rect['x1']}),({rect['y0']},{rect['y1']})) ")
 
     def print_curves(self, page):
-        if n_curve := len(page.curves) > 0:
+        n_curve = len(page.curves)
+        if n_curve > 0:
             print(f"curves {n_curve}", end=" | ")
             for curve in page.curves[:self.max_curve]:
                 print(f"keys: {curve.keys()}")
@@ -1143,7 +1250,8 @@ class PDFDebug:
         write_image = False
         resolution = 400  # may be better
             # see https://github.com/euske/pdfminer/blob/master/pdfminer/pdftypes.py
-        if (n_image := len(page.images)) > 0:
+        n_image = len(page.images)
+        if n_image > 0:
             print(f"images {n_image}", end=" |\n")
             for i, image in enumerate(page.images[:maximage]):
 
@@ -1188,8 +1296,8 @@ class PDFDebug:
 
     def print_tables(self, page, odir=None):
         tables = page.find_tables()
-
-        if n_table := len(tables) > 0:
+        n_table = len(tables)
+        if n_table > 0:
             print(f"tables {n_table}", end=" | ")
             print(f"table_dir {tables[0].__dir__()}")
             for i, table in enumerate(tables[:self.max_table]):
@@ -1219,7 +1327,8 @@ class PDFDebug:
         return f"{int(bbox[0])}_{int(bbox[2])}_{int(bbox[1])}_{int(bbox[3])}"
 
     def print_hyperlinks(self, page):
-        if n_hyper := len(page.hyperlinks) > 0:
+        n_hyper = len(page.hyperlinks)
+        if n_hyper > 0:
             print(f"hyperlinks {n_hyper}", end=" | ")
             for hyperlink in page.hyperlinks:
                 print(f"hyperlink {hyperlink.values()}")
@@ -1258,7 +1367,8 @@ class PDFDebug:
         and there are 34 (in a TableOfContents) and they work
 
         """
-        if n_annot := len(page.annots) > 0:
+        n_annot = len(page.annots)
+        if n_annot > 0:
             print(f"annots {n_annot}", end=" | ")
             for annot in page.annots:
                 print(f"annot: {annot.items()}")
@@ -1759,6 +1869,10 @@ def main(argv=None):
     """
     print(f"running PDFArgs main")
     pdf_args = PDFArgs()
+    parse_and_process_1(pdf_args)
+
+
+def parse_and_process_1(pdf_args):
     try:
         pdf_args.parse_and_process()
         pdf_args.convert_write()
